# coding: utf-8
# author: Gabriel Couture
IDENTIFIER = '22dcf059-8fd3ade7-efb39ca3-7f46b248-0200abc9'
FILE_SIZE = 1506520
CREATION_DATE = '20100301'
CREATION_TIME = '170155'
PARENT_SERIES_IDENTIFIER = 'c4c1fcc9-ae63f793-40cbcf25-fbd3efe5-ad72ff06'

INFORMATION = {
    'FileSize': FILE_SIZE,
    'FileUuid': '3fd3d1c3-0a10-4832-b20a-d48ca7116835',
    'ID': IDENTIFIER,
    'IndexInSeries': 1,
    'MainDicomTags': {
        'ImageOrientationPatient': '1\\0\\0\\0\\1\\0',
        'ImagePositionPatient': '-223.9880065918\\-158.08148193359\\-117.78499603271',
        'InstanceCreationDate': CREATION_DATE,
        'InstanceCreationTime': CREATION_TIME,
        'InstanceNumber': '1',
        'NumberOfFrames': '75',
        'SOPInstanceUID': '2.16.840.1.113669.2.931128.981194414.20100301170155.668389'
    },
    'ParentSeries': PARENT_SERIES_IDENTIFIER,
    'Type': 'Instance'
}
