# coding: utf-8
# author: Gabriel Couture

IDENTIFIER = '118bc493-b3b3172a-082119bd-f6802ec3-81695613'
PARENT_PATIENT_IDENTIFIER = 'e34c28ce-981b0e5c-2a481559-cf0d5fbe-053335f8'
PATIENT_MAIN_INFORMATION = {
    'PatientBirthDate': '',
    'PatientID': '03HDQ000',
    'PatientName': 'MR-R',
    'PatientSex': 'M'
}
DATE = '20100223'
REFERRING_PHYSICIAN_NAME = ''
ID = ''
TIME = ''
SERIES = [
    'c4c1fcc9-ae63f793-40cbcf25-fbd3efe5-ad72ff06',
    'e2a7df26-99673e0f-05aa84cd-c89677c0-634a2a96',
    '60108266-ece4d8f7-7b028286-a7b61f25-c6d33f0b'
]
INFORMATION = {
    'ID': IDENTIFIER,
    'IsStable': False,
    'LastUpdate': '20190913T185658',
    'MainDicomTags': {
        'AccessionNumber': '',
        'ReferringPhysicianName': '',
        'StudyDate': DATE,
        'StudyID': ID,
        'StudyInstanceUID': '1.2.840.113619.2.55.3.671782272.419.1266452812.128',
        'StudyTime': TIME
    },
    'ParentPatient': PARENT_PATIENT_IDENTIFIER,
    'PatientMainDicomTags': PATIENT_MAIN_INFORMATION,
    'Series': SERIES,
    'Type': 'Study'
}
