# coding: utf-8
# author: Gabriel Couture

IDENTIFIER = 'c4c1fcc9-ae63f793-40cbcf25-fbd3efe5-ad72ff06'
INSTANCES = ['22dcf059-8fd3ade7-efb39ca3-7f46b248-0200abc9']
MODALITY = 'RTDOSE'
MANUFACTURER = 'ADAC'
PARENT_STUDY = '118bc493-b3b3172a-082119bd-f6802ec3-81695613'
INFORMATION = {
    'ExpectedNumberOfInstances': None,
    'ID': IDENTIFIER,
    'Instances': INSTANCES,
    'IsStable': False,
    'LastUpdate': '20190916T131815',
    'MainDicomTags': {
        'ImageOrientationPatient': '1\\0\\0\\0\\1\\0',
        'Manufacturer': MANUFACTURER,
        'Modality': MODALITY,
        'SeriesInstanceUID': '2.16.840.1.113669.2.931128.981194414.20100301170148.767969',
        'SeriesNumber': '',
        'StationName': 'pinnc-2'
    },
    'ParentStudy': PARENT_STUDY,
    'Status': 'Unknown',
    'Type': 'Series'
}
